//**********************************************************************
// 	Project: FIR project
//	File: sim_input_gen.v
// 	Description: 
//	Author: xxx
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	| xxx		| xx/xx/20xx	| Initial version
//**********************************************************************

module sim_input_gen (
		clk,
		rstn, 
		
		rfi_i, 
		
		sim_valid_o,
		sim_sync_o,
		sim_data_o
);

//**********************************************************************
// --- Parameter
//**********************************************************************
	parameter DATA_WIDTH = 16;
	parameter NUM_CHN = 4;
	parameter NUM_CYCLE = 4;
	
	localparam CYCLE_WIDTH = (NUM_CYCLE>1)? $clog2(NUM_CYCLE):1;
	localparam CHN_WIDTH = (NUM_CHN>1)? $clog2(NUM_CHN):1;

    localparam LIMITE_MAX = 32767;
	localparam LIMITE_MIN = -32768;

//**********************************************************************
// --- Input/Output Declaration
//**********************************************************************
// --- input
    input                       clk;
    input                       rstn;
	input 					    rfi_i;

// --- output
	output reg 					sim_valid_o;
	output reg 					sim_sync_o;
	output reg [DATA_WIDTH-1:0] sim_data_o;

//**********************************************************************
// --- Internal Signal Declaration
//**********************************************************************
	reg rfi_r0;
	wire rfi_posedge;
	
	reg [CYCLE_WIDTH:0] cnt_cycle;
	reg [CHN_WIDTH-1:0] cnt_chn;
	
	reg flag_incr, flag_decr;
	
	wire					 sim_valid;
	wire					 sim_sync;
	reg [DATA_WIDTH-1:0]    sim_data;


//**********************************************************************
// --- Main core
//**********************************************************************
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			rfi_r0 <= 0;
		end 
		else begin
			rfi_r0 <= rfi_i;
		end
	end
	assign rfi_posedge = rfi_i & ~rfi_r0;
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			cnt_cycle <= NUM_CYCLE;
		end
		else if(rfi_posedge == 1'b1 || (rfi_i==1'b1 && cnt_cycle == NUM_CYCLE-1))begin
			cnt_cycle <= 0;
		end
		else if((cnt_cycle == NUM_CYCLE || cnt_cycle == NUM_CYCLE-1) && cnt_chn == NUM_CHN-1) begin
			cnt_cycle <= cnt_cycle;
		end
		else begin
			cnt_cycle <= cnt_cycle+1'b1;
		end
	end

	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			cnt_chn <= NUM_CHN-1;
		end
		else if(rfi_posedge == 1'b1 || (rfi_i==1'b1 && cnt_chn == NUM_CHN-1 && cnt_cycle == NUM_CYCLE-1))begin
			cnt_chn <= 0;
		end
		else if(cnt_chn == NUM_CHN-1) begin
			cnt_chn <= cnt_chn;
		end
		else if(cnt_cycle == NUM_CYCLE-1)begin
			cnt_chn <= cnt_chn+1'b1;
		end
	end
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			flag_incr <= 1'b1;
			flag_decr <= 1'b0;
		end
		else if(sim_data == LIMITE_MIN)begin
			flag_incr <= 1'b1;
			flag_decr <= 1'b0;
		end
		else if(sim_data == LIMITE_MAX)begin
			flag_incr <= 1'b0;
			flag_decr <= 1'b1;
		end
	end
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			sim_data <= 0;
		end
		else if(cnt_cycle==0 && flag_incr==1'b1)begin
			sim_data <= sim_data + 1'b1;
		end
		else if(cnt_cycle==0 && flag_decr == 1'b1) begin
			sim_data <= sim_data - 1'b1;
		end
	end
	
	assign sim_valid = (cnt_cycle == 0)?1'b1:1'b0;
	assign sim_sync = (cnt_cycle == 0 && cnt_chn == 0)?1'b1:1'b0;
	

	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			sim_data_o <= 0;
		end
		else if(sim_valid)begin
			sim_data_o <= sim_data;
		end
	end

	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			sim_valid_o <= 0;
			sim_sync_o <= 0;
		end
		else begin
			sim_valid_o <= sim_valid;
			sim_sync_o <= sim_sync;
		end
	end

endmodule