//**********************************************************************
// 	Project: xxx project
//	File: sim_output_storage.v
// 	Description: 
//	Author: Noa
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	| Noa		| xx/xx/20xx	| Initial version
//**********************************************************************

module sim_output_storage (
		clk,
		rstn, 
		
		sim_valid_i,
		sim_sync_i,
		sim_data_i,
		
		sim_index_o,
		sim_data_o
);

//**********************************************************************
// --- Parameter
//**********************************************************************
	parameter DATA_WIDTH = 16;
	parameter MEM_DEPTH = 1024;
	parameter NUM_CHN = 4;
	parameter CHN_SEL = 0;
	
	
	localparam CHN_WIDTH = (NUM_CHN>1)? $clog2(NUM_CHN):1;
	localparam MEM_WIDTH = (MEM_DEPTH>1)? $clog2(MEM_DEPTH):1;


//**********************************************************************
// --- Input/Output Declaration
//**********************************************************************
// --- input
    input                       clk;
    input                       rstn;
	input  						sim_valid_i;
	input  						sim_sync_i;
	input  [DATA_WIDTH-1:0] 	sim_data_i;


// --- output
	output reg [MEM_WIDTH-1:0]	sim_index_o;
	output wire [DATA_WIDTH-1:0] sim_data_o;

//**********************************************************************
// --- Internal Signal Declaration
//**********************************************************************
	reg 					ram_din_we;
	reg [MEM_WIDTH-1:0] 	ram_din_addra;
	reg [DATA_WIDTH-1:0] 	ram_din_data;
	
	reg [MEM_WIDTH-1:0] 	ram_dout_addrb;
	wire [DATA_WIDTH-1:0] 	ram_dout_data;
	
	reg  					sim_valid;
	reg  					sim_sync;
	reg  [DATA_WIDTH-1:0] 	sim_data;
	
	reg [CHN_WIDTH-1:0] 	cnt_chn;
	wire 					sel_valid;

//**********************************************************************
// --- Main core
//**********************************************************************
	spram spram_dout_inst(			
		.rstn_i		( rstn							),
		.clk_a_i	( clk							), 
		.wr_a_i		( ram_din_we					),
		.addr_a_i	( ram_din_addra					),
		.data_a_i	( ram_din_data					),
					
		.clk_b_i	( clk							),
		.rd_b_i		( 1'b1							),
		.addr_b_i	( ram_dout_addrb				),
		.data_b_o	( ram_dout_data					)
	);
	defparam spram_dout_inst.DATA_WIDTH = DATA_WIDTH;
	defparam spram_dout_inst.MEM_DEPTH = MEM_DEPTH;
	
// --- cnt_chn
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			sim_valid <= 0;
			sim_sync <= 0;
			sim_data <= 0;
		end
		else begin
			sim_valid <= sim_valid_i;
			sim_sync <=  sim_sync_i;
			sim_data <=  sim_data_i;
		
		end
	end

	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			cnt_chn <= NUM_CHN-1;
		end
		else if(sim_valid_i) begin
			if(cnt_chn == NUM_CHN-1) begin
				cnt_chn <= 0;
			end
			else if(sim_valid_i) begin
				cnt_chn <= cnt_chn+1'b1;
			end
		end
		
	end
	assign sel_valid = (cnt_chn == CHN_SEL && sim_valid == 1'b1)? 1'b1:1'b0;
	
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			ram_din_we <= 0;
		end
		else if(ram_din_addra == MEM_DEPTH-1) begin
			ram_din_we <= 0;
		end
		else begin
			ram_din_we <= sel_valid;
		end
	end 
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			ram_din_addra <= 0;
		end
		else if(sel_valid) begin
			if(ram_din_addra == MEM_DEPTH-1) 
				ram_din_addra <= ram_din_addra;
			else 
				ram_din_addra <= ram_din_addra + 1'b1;
		end
	end 
	
	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			ram_din_data <= 0;
		end
		else begin
			ram_din_data <= sim_data;
		end
	end 

	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			ram_dout_addrb <= MEM_DEPTH-1;
		end
		else if(sel_valid) begin
			if(ram_dout_addrb == MEM_DEPTH-1) 
				ram_dout_addrb <= 0;
			else
				ram_dout_addrb <= ram_dout_addrb + 1'b1;
		end
	end 


	always @(posedge clk or negedge rstn) begin
		if(!rstn) begin
			sim_index_o <= 0;
		end
		else begin
			sim_index_o <= ram_dout_addrb;
		end
	end 
	assign sim_data_o = ram_dout_data;

endmodule