//**********************************************************************
// 	Project: xxx project
//	File: spram.v
// 	Description: 
//	Author: Noa
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	| Noa		| xx/xx/20xx	| Initial version
//**********************************************************************
`define AUTOMATIC_MEMORY

module spram (
		rstn_i, 
		
		clk_a_i, 
		wr_a_i,
		addr_a_i,
		data_a_i,
		
		clk_b_i,
		rd_b_i,
		addr_b_i,
	 	data_b_o
);

//**********************************************************************
// --- Parameter
//**********************************************************************
	parameter DATA_WIDTH = 8;
	parameter MEM_DEPTH = 512;
	localparam ADDR_WIDTH = (MEM_DEPTH>1)? $clog2(MEM_DEPTH):1;

//**********************************************************************
// --- Input/Output Declaration
//**********************************************************************
// --- input ---	
	input wire 							rstn_i; 
	input wire 							clk_a_i; 
	input wire 							wr_a_i; 
	input wire [ADDR_WIDTH-1:0]			addr_a_i;
	input wire [DATA_WIDTH-1:0]			data_a_i;
		  
	input wire 							clk_b_i; 
	input wire 							rd_b_i; 
	input wire [ADDR_WIDTH-1:0]			addr_b_i;
// --- output ---	
	output reg [DATA_WIDTH-1:0] 		data_b_o;
	
//**********************************************************************
// --- Internal Signal Declaration
//**********************************************************************
`ifdef EBR_MEMORY
    reg [DATA_WIDTH-1:0] mem [0:MEM_DEPTH-1] /* synthesis syn_ramstyle = "block_ram" */;
`endif
`ifdef DISTRIBUTED_MEMORY
   reg [DATA_WIDTH-1:0] mem [0:MEM_DEPTH-1] /* synthesis syn_ramstyle = "distributed_ram" */;
`endif
`ifdef REG_MEMORY
     reg [DATA_WIDTH-1:0] mem[0:MEM_DEPTH-1] /* synthesis syn_ramstyle = "registers" */;
`endif
`ifdef AUTOMATIC_MEMORY
    reg [DATA_WIDTH-1:0] mem [0:MEM_DEPTH-1];
`endif

	integer i_mem; 
	
//**********************************************************************
// --- Main core
//**********************************************************************
	always @(posedge clk_a_i or negedge rstn_i) begin
		if(!rstn_i) begin
			for(i_mem=0; i_mem<MEM_DEPTH; i_mem=i_mem+1)
				mem[i_mem] <= 0;
		end
		else if(wr_a_i) begin
			mem[addr_a_i] <= data_a_i;
		end
	end

	always @(posedge clk_b_i or negedge rstn_i) begin
		if(!rstn_i) begin
			data_b_o <= 0;
		end
		else if(rd_b_i)begin
			data_b_o <= mem[addr_b_i];
		end
	end

endmodule